function WeaponSelectionGui:_layout_category_tabs()
	local category_tabs_params = {
		tab_align = "center",
		name = "category_tabs",
		initial_tab_idx = 1,
		tab_height = 64,
		y = 0,
		x = 0,
		on_click_callback = callback(self, self, "on_weapon_category_selected"),
		parent_control_ref = self,
		tabs_params = {
			{
				name = "tab_primary",
				text = self:translate("menu_weapons_tab_category_primary", true),
				callback_param = WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID,
				breadcrumb = {
					check_callback = callback(managers.weapon_skills, managers.weapon_skills, "has_weapon_breadcrumbs", {
						weapon_category = WeaponInventoryManager.CATEGORY_NAME_PRIMARY
					})
				}
			},
			{
				name = "tab_secondary",
				text = self:translate("menu_weapons_tab_category_secondary", true),
				callback_param = WeaponInventoryManager.BM_CATEGORY_SECONDARY_ID,
				breadcrumb = {
					check_callback = callback(managers.weapon_skills, managers.weapon_skills, "has_weapon_breadcrumbs", {
						weapon_category = WeaponInventoryManager.CATEGORY_NAME_SECONDARY
					})
				}
			},
			{
				name = "tab_grenades",
				text = self:translate("menu_weapons_tab_category_grenades", true),
				callback_param = WeaponInventoryManager.BM_CATEGORY_GRENADES_ID
			},
			{
				name = "tab_melee",
				text = self:translate("menu_weapons_tab_category_melee", true),
				breadcrumb = {
					category = BreadcrumbManager.CATEGORY_WEAPON_MELEE
				},
				callback_param = WeaponInventoryManager.BM_CATEGORY_MELEE_ID
			}
		}
	}
	self._list_tabs = self._weapon_selection_panel:tabs(category_tabs_params)
	self._selected_weapon_category_id = WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID

	-- Changed menu
	self:_layout_filter_tabs_primary()
	self:_layout_filter_tabs_secondary()
	self:_layout_filter_tabs_all()
end

function WeaponSelectionGui:_layout_filter_tabs_primary()
	self._equippable_filters_tabs = self._weapon_selection_panel:tabs({
		name = "equippable_filters_tabs",
		tab_width = 110,
		initial_tab_idx = 1,
		tab_align = "center",
		tab_height = 64,
		y = 54,
		x = 0,
		icon = tweak_data.gui.icons.ico_filter,
		item_class = RaidGUIControlTabFilter,
		on_click_callback = callback(self, self, "on_click_filter_equippable"),
		tabs_params = {
			{
				name = "tab_all",
				callback_param = "all",
				text = self:translate("menu_filter_all", true)
			},
			{
				name = "tab_smg",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_SMG,
				text = self:translate("weapon_catagory_smg_short", true)
			},
			{
				name = "tab_assault_rifle",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_ASSAULT_RIFLE,
				text = self:translate("weapon_catagory_assault_rifle_short", true)
			},
			{
				name = "tab_lmg",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_LMG,
				text = self:translate("weapon_catagory_lmg_short", true)
			},
			{
				name = "tab_snp",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_SNP,
				text = self:translate("weapon_catagory_snp_short", true)
			},
			{
				name = "tab_shotgun",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_SHOTGUN,
				text = self:translate("weapon_catagory_shotgun", true)
			},
		}
	})
	self._selected_filter = "all"
end

function WeaponSelectionGui:_layout_filter_tabs_secondary()
	self._equippable_filters_tabs_secondary = self._weapon_selection_panel:tabs({
		name = "equippable_filters_tabs_secondary",
		tab_width = 110,
		initial_tab_idx = 1,
		tab_align = "center",
		tab_height = 64,
		y = 54,
		x = 0,
		icon = tweak_data.gui.icons.ico_filter,
		item_class = RaidGUIControlTabFilter,
		on_click_callback = callback(self, self, "on_click_filter_equippable"),
		tabs_params = {
			{
				name = "tab_all",
				callback_param = "all",
				text = self:translate("menu_filter_all", true)
			},
			{
				name = "tab_pistol",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_PISTOL,
				text = self:translate("weapon_catagory_pistol", true)
			},
			{
				name = "tab_shotgun",
				callback_param = WeaponTweakData.WEAPON_CATEGORY_SHOTGUN,
				text = self:translate("weapon_catagory_shotgun", true)
			},
		}
	})
	self._equippable_filters_tabs_secondary:hide()
end

function WeaponSelectionGui:_layout_filter_tabs_all()
	self._equippable_filters_tabs_all = self._weapon_selection_panel:tabs({
		name = "equippable_filters_tabs_all",
		tab_width = 110,
		initial_tab_idx = 1,
		tab_align = "center",
		tab_height = 64,
		y = 54,
		x = 0,
		icon = tweak_data.gui.icons.ico_filter,
		item_class = RaidGUIControlTabFilter,
		on_click_callback = callback(self, self, "on_click_filter_equippable"),
		tabs_params = {
			{
				name = "tab_all",
				callback_param = "all",
				text = self:translate("menu_filter_all", true)
			},
		}
	})
	self._equippable_filters_tabs_all:hide()
end

function WeaponSelectionGui:data_source_weapon_list()
	local result = {}
	local owned_weapons = {}

	if self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID then
		owned_weapons = managers.weapon_inventory:get_owned_weapons(WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID)
	elseif self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_SECONDARY_ID then
		owned_weapons = managers.weapon_inventory:get_owned_weapons(WeaponInventoryManager.BM_CATEGORY_SECONDARY_ID)
	elseif self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_MELEE_ID then
		owned_weapons = self._cached_owned_melee_weapons or managers.weapon_inventory:get_owned_melee_weapons()
	elseif self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_GRENADES_ID then
		owned_weapons = managers.weapon_inventory:get_owned_grenades()
	end

	local equipped_weapon_id = nil

	if owned_weapons then
		for _, weapon_data in pairs(owned_weapons) do
			-- Changed condition here to check for weapon category
			if self._selected_filter == 'all' or self._selected_filter == tweak_data.weapon[weapon_data.weapon_id].category then
				if self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID or self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_SECONDARY_ID then
					local breadcrumb_category = nil

					if self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID then
						breadcrumb_category = BreadcrumbManager.CATEGORY_WEAPON_PRIMARY
						equipped_weapon_id = managers.weapon_inventory:get_equipped_primary_weapon_id()
					else
						breadcrumb_category = BreadcrumbManager.CATEGORY_WEAPON_SECONDARY
						equipped_weapon_id = managers.weapon_inventory:get_equipped_secondary_weapon_id()
					end

					local weapon_category = managers.weapon_inventory:get_weapon_category_name_by_bm_category_id(self
					._selected_weapon_category_id)
					local breadcrumb = nil

					if weapon_data.unlocked then
						breadcrumb = {
							check_callback = callback(managers.weapon_skills, managers.weapon_skills,
								"has_weapon_breadcrumbs", {
								weapon_category = weapon_category,
								weapon_id = weapon_data.weapon_id
							}),
							category = breadcrumb_category,
							identifiers = {
								weapon_data.weapon_id
							}
						}
					end

					if self._selected_weapon_id == weapon_data.weapon_id or equipped_weapon_id == weapon_data.weapon_id then
						table.insert(result, {
							selected = true,
							text = self:translate(tweak_data.weapon[weapon_data.weapon_id].name_id, false),
							value = weapon_data,
							breadcrumb = breadcrumb
						})
					else
						table.insert(result, {
							text = self:translate(tweak_data.weapon[weapon_data.weapon_id].name_id, false),
							value = weapon_data,
							breadcrumb = breadcrumb
						})
					end
				elseif self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_MELEE_ID then
					equipped_weapon_id = managers.weapon_inventory:get_equipped_melee_weapon_id()
					local breadcrumb = {
						category = BreadcrumbManager.CATEGORY_WEAPON_MELEE,
						identifiers = {
							weapon_data.weapon_id
						}
					}

					if self._selected_weapon_id == weapon_data.weapon_id or equipped_weapon_id == weapon_data.weapon_id then
						table.insert(result, {
							selected = true,
							text = self:translate(tweak_data.blackmarket.melee_weapons[weapon_data.weapon_id].name_id,
								false),
							value = weapon_data,
							breadcrumb = breadcrumb
						})
					else
						table.insert(result, {
							text = self:translate(tweak_data.blackmarket.melee_weapons[weapon_data.weapon_id].name_id,
								false),
							value = weapon_data,
							breadcrumb = breadcrumb
						})
					end
				elseif self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_GRENADES_ID then
					equipped_weapon_id = managers.weapon_inventory:get_equipped_grenade_id()

					if self._selected_weapon_id == weapon_data.weapon_id or equipped_weapon_id == weapon_data.weapon_id then
						table.insert(result, {
							selected = true,
							text = self:translate(tweak_data.projectiles[weapon_data.weapon_id].name_id, false),
							value = weapon_data
						})
					else
						table.insert(result, {
							text = self:translate(tweak_data.projectiles[weapon_data.weapon_id].name_id, false),
							value = weapon_data
						})
					end
				end
			end
		end
	end

	if self._selected_weapon_category_id ~= WeaponInventoryManager.BM_CATEGORY_MELEE_ID then
		local class_name = managers.skilltree:get_character_profile_class()

		table.sort(result, function(l, r)
			local l_level = tweak_data.skilltree:get_weapon_unlock_level(l.value.weapon_id, class_name) or 100000
			local r_level = tweak_data.skilltree:get_weapon_unlock_level(r.value.weapon_id, class_name) or 100000

			if l_level ~= r_level then
				return l_level < r_level
			end

			return l.text < r.text
		end)

		return result
	end

	table.sort(result, function(l, r)
		if l.value.unlocked and not r.value.unlocked then
			return true
		elseif not l.value.unlocked and r.value.unlocked then
			return false
		end

		return l.text < r.text
	end)

	return result
end

-- Completely changed this whole function because otherwise it would crash the game
-- Resetting selection to first item on the list if we switched to different category that isn't 'all'
function WeaponSelectionGui:_reselect_weapons_in_list()
	local selected_weapon_data = nil

	if self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID or self._selected_weapon_category_id == WeaponInventoryManager.BM_CATEGORY_SECONDARY_ID then
		local selected_weapon_id = self._selected_weapon_id
		if self._selected_filter == 'all' or self._selected_filter == tweak_data.weapon[selected_weapon_id].category then
			selected_weapon_data = self._weapon_list:selected_item():data()
		end
	end

	self._weapon_list:refresh_data()

	if selected_weapon_data then
		self._weapon_list:select_item_by_value(selected_weapon_data.value)
	else
		self._weapon_list:select_item_by_index(1, false, true)
	end
end

-- Hiding menu for everything other than primary weapons
Hooks:PreHook(WeaponSelectionGui, "on_weapon_category_selected", "better_categories_in_weapon_selection_on_weapon_category_selected", function(self, selected_category)
	if selected_category == WeaponInventoryManager.BM_CATEGORY_PRIMARY_ID then
		self._equippable_filters_tabs:show()
		self._equippable_filters_tabs_secondary:hide()
		self._equippable_filters_tabs_all:hide()

		self._equippable_filters_tabs:set_selected_index(1)
	elseif selected_category == WeaponInventoryManager.BM_CATEGORY_SECONDARY_ID then
		self._equippable_filters_tabs:hide()
		self._equippable_filters_tabs_secondary:show()
		self._equippable_filters_tabs_all:hide()

		self._equippable_filters_tabs_secondary:set_selected_index(1)
	else
		self._equippable_filters_tabs:hide()
		self._equippable_filters_tabs_secondary:hide()
		self._equippable_filters_tabs_all:show()

		self._equippable_filters_tabs_all:set_selected_index(1)
	end
end)

function WeaponSelectionGui:_on_equipable_tab_left()
	if self._equippable_filters_tabs:visible() then
		self._equippable_filters_tabs:_move_left()
	end
	if self._equippable_filters_tabs_secondary:visible() then
		self._equippable_filters_tabs_secondary:_move_left()
	end

	return true, nil
end

function WeaponSelectionGui:_on_equipable_tab_right()
	if self._equippable_filters_tabs:visible() then
		self._equippable_filters_tabs:_move_right()
	end
	if self._equippable_filters_tabs_secondary:visible() then
		self._equippable_filters_tabs_secondary:_move_right()
	end

	return true, nil
end
